local hcTk = HCToolkit
local UnitGUID = UnitGUID

function HCToolkit:OnInitialize()
    -- Called when the addon is loaded
    -- Register Frames
    HCToolkitFrame = CreateFrame("Frame", "HCToolkitFrame", UIParent);
end

function HCToolkit:OnEnable()
    -- Called when the addon is enabled
    -- Register Events
    RegisterAddonMessagePrefix("HCToolkit")
    RegisterAddonMessagePrefix("HCToolkitRoster")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    self:RegisterEvent("ENCOUNTER_START")
    self:RegisterEvent("ENCOUNTER_END")
    self:RegisterEvent("GROUP_JOINED")
    self:RegisterEvent("GROUP_ROSTER_UPDATE")
    self:RegisterEvent("PLAYER_ENTERING_WORLD")
    self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
    self:RegisterChatCommand("hctk", "ChatCommand")
    self:RegisterChatCommand("HCToolkit", "ChatCommand")
    self:RegisterComm("HCToolkit", "CoreComm")
    self:RegisterComm("HCToolkitRoster", "RosterComm")

    -- Setup Addon
    hcTk.InitDB()
    hcTk.GroupInit()
    hcTk.EncounterInit()
    hcTk.FramesInit()
end

function HCToolkit:OnDisable()
  -- Called when the addon is disabled
end

function HCToolkit:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
    if (hcTk.EncounterId ~= nil) then
        hcTk.RouteCombatLog(hcTk.EncounterId, ...)
    end
end

function HCToolkit:ENCOUNTER_START(event, encounterId)
    hcTk.DebugPrint("Encounter Start: " .. encounterId)
    hcTk.EncounterId = encounterId
    hcTk.FrameDemoMode(HCToolkitGroupFrame, false)
    hcTk.FrameDemoMode(HCToolkitPersonalFrame, false)
    hcTk.RouteEncounterStart(encounterId)
end

function HCToolkit:ENCOUNTER_END(event, encounterId)
    hcTk.EncounterId = nil
    hcTk.RouteEncounterEnd(encounterId)
    hcTk.FrameDemoMode(HCToolkitGroupFrame, false)
    hcTk.FrameDemoMode(HCToolkitPersonalFrame, false)
end

function HCToolkit:GROUP_JOINED()
    hcTk.SpecializationSend(UnitGUID("player"))
    hcTk.SpecializationRequest()
end

function HCToolkit:GROUP_ROSTER_UPDATE()
    hcTk.DebugPrint("HC Group Roster Update")
    hcTk.GroupUpdate()
end

function HCToolkit:PLAYER_ENTERING_WORLD()
    hcTk.DebugPrint("HC Login")
    hcTk.GroupFill()
    hcTk.SpecializationSend(UnitGUID("player"))
    hcTk.SpecializationRequest()
end

function HCToolkit:PLAYER_SPECIALIZATION_CHANGED()
    hcTk.SpecializationSend(UnitGUID("player"))
end

function HCToolkit:ChatCommand(input)
    if (string.find(input, "version")) then
        hcTk:Print("Version: " .. hcTk.Constants.Version)
    else
        InterfaceOptionsFrame_OpenToCategory(hcTk.OptionsFrame)
        InterfaceOptionsFrame_OpenToCategory(hcTk.OptionsFrame)
    end
end

function HCToolkit:CoreComm(prefix, input, distribution, sender)
end

function HCToolkit:RosterComm(prefix, input, distribution, sender)
    local status, data = hcTk:Deserialize(input);
    if (data["type"] == "SpecializationSend" and data["guid"] ~= nil and data["specId"] ~= nil) then
        hcTk.UnitUpdateRole(data["guid"], data["specId"])
    elseif (data["type"] == "SpecializationRequest") then
        hcTk.SpecializationSend(UnitGUID("player"))
    end
end