local hcTk = HCToolkit

function hcTk.AvatarEncounterStart()
    hcTk.Encounter["DarkMarks"] = {}
    for k, v in pairs(hcTk.Group) do
        v["DarkMarkExpiration"] = 0
    end
end

function hcTk.AvatarEncounterEnd()
    HCToolkitGroupFrame:SetScript("OnUpdate", nil)
    HCToolkitPersonalFrame:SetScript("OnUpdate", nil)

    hcTk.Encounter["DarkMarks"] = nil
    for k, v in pairs(hcTk.Group) do
        v["DarkMarkExpiration"] = nil
    end
end

function hcTk.AvatarCombatLog(...)
    local msg = select(2, ...)
    if (hcTk.AvatarCombatLogFns[msg] ~= nil) then
        hcTk.AvatarCombatLogFns[msg](...)
    end
end

function hcTk.AvatarSpellAuraApplied(...)
    local destGUID, _, _, _, spellId = select(8, ...)
    if (spellId == 239739) then -- Dark Mark
        hcTk.AvatarDarkMarkHandler(destGUID)
    elseif (spellId == 242017) then -- Black Winds
        hcTk.AvatarBlackWindsHandler(destGUID)
    end
end

function hcTk.AvatarSpellCastSuccess(...)

end

function hcTk.AvatarUnitDied(...)

end

function hcTk.AvatarBlackWindsHandler(destGUID)
    if (destGUID == hcTk.PlayerGUID) then
        PlaySoundFile("Interface\\AddOns\\HCToolkit\\Media\\ThatsWrong.mp3", "Master")
    end
end

function hcTk.AvatarDarkMarkHandler(destGUID)
    hcTk.Encounter["DarkMarks"][#hcTk.Encounter["DarkMarks"] + 1] = destGUID
    hcTk.Group[destGUID]["DarkMarkExpiration"] = GetTime() + 6 + (2 * (#hcTk.Encounter["DarkMarks"] - 1)) -- Mark #1: 6s, Mark #2: 8s, Mark #3: 10s

    if (#hcTk.Encounter["DarkMarks"] == 3 and hcTk.GenericGetBossOption("TOS", "Avatar", "DarkMarksGroup")) then -- Dark Marks Complete, Trigger Printing
        HCToolkitGroupFrame:SetScript("OnUpdate", hcTk.AvatarPrintGroupDarkMarks)
    end

    if (hcTk.PlayerGUID == destGUID and hcTk.GenericGetBossOption("TOS", "Avatar", "DarkMarkPersonal")) then
        HCToolkitPersonalFrame:SetScript("OnUpdate", hcTk.AvatarPrintPersonalDarkMark)
    end

    if (#hcTk.Encounter["DarkMarks"] == 3) then
        C_Timer.After(11, hcTk.AvatarWipeDarkMarks)
    end
end

function hcTk.AvatarPrintPersonalDarkMark()
    local positions = { [1] = "Left", [2] = "Middle", [3] = "Right" }
    local markers = { [1] = 3, [2] = 4, [3] = 5 }

    for k, m in ipairs(hcTk.Encounter["DarkMarks"]) do
        if (hcTk.PlayerGUID == m) then
            if (GetTime() > hcTk.Group[m]["DarkMarkExpiration"]) then
                HCToolkitPersonalFrame:SetScript("OnUpdate", nil)
                hcTk.FrameUpdateText(HCToolkitPersonalFrame, "")
            else
                hcTk.FrameUpdateText(HCToolkitPersonalFrame, string.format("%s%s: %s - %.1f", hcTk.MarkerIdToString(markers[k]), positions[k], hcTk.UnitPlayerString(hcTk.Group[m]), math.max(0, hcTk.Group[m]["DarkMarkExpiration"] - GetTime())))
            end
        end
    end
end

function hcTk.AvatarPrintGroupDarkMarks()
    local positions = { [1] = "Left", [2] = "Middle", [3] = "Right" }
    local markers = { [1] = 3, [2] = 4, [3] = 5 }

    local str = ""
    for k, m in ipairs(hcTk.Encounter["DarkMarks"]) do
        str = str .. string.format("%s%s: %s - %.1f\n", hcTk.MarkerIdToString(markers[k]), positions[k], hcTk.UnitPlayerString(hcTk.Group[m]), math.max(0, hcTk.Group[m]["DarkMarkExpiration"] - GetTime()))
    end
    
    if (GetTime() > hcTk.Group[hcTk.Encounter["DarkMarks"][#hcTk.Encounter["DarkMarks"]]]["DarkMarkExpiration"]) then
        HCToolkitGroupFrame:SetScript("OnUpdate", nil)
        hcTk.FrameUpdateText(HCToolkitGroupFrame, "")
    else
        hcTk.FrameUpdateText(HCToolkitGroupFrame, str)
    end
end

function hcTk.AvatarWipeDarkMarks()
    hcTk.Encounter["DarkMarks"] = {}
    for k, v in pairs(hcTk.Group) do
        v["DarkMarkExpiration"] = 0
    end
end

hcTk.AvatarCombatLogFns = {
    ["SPELL_AURA_APPLIED"] = hcTk.AvatarSpellAuraApplied,
    ["SPELL_CAST_SUCCESS"] = hcTk.AvatarSpellCastSuccess,
    ["UNIT_DIED"] = hcTk.AvatarUnitDied
}